@@ -280,14 +280,20 @@ emit_change_event (gpointer key,
 static void
 fail_job (GVfsJob *job, LIBMTP_mtpdevice_t *device)
 {
+  const char *text;
   LIBMTP_error_t *error = LIBMTP_Get_Errorstack (device);
 
+  if (error) {
+    text = g_strrstr (error->error_text, ":") + 1;
+  } else {
+    text = _("Unknown error.");
+  }
   g_vfs_job_failed (job, G_IO_ERROR,
                     g_vfs_job_is_cancelled (job) ?
                       G_IO_ERROR_CANCELLED :
                       G_IO_ERROR_FAILED,
                     _("libmtp error: %s"),
-                    g_strrstr (error->error_text, ":") + 1);
+                    text);
 
   LIBMTP_Clear_Errorstack (device);
 }
@@ -1209,19 +1215,24 @@ do_pull (GVfsBackend *backend,
                               _("Can't recursively copy directory"));
     goto exit;
   } else {
-      MtpProgressData mtp_progress_data;
-      mtp_progress_data.progress_callback = progress_callback;
-      mtp_progress_data.progress_callback_data = progress_callback_data;
-      mtp_progress_data.job = G_VFS_JOB (job);
-      int ret = LIBMTP_Get_File_To_File (device,
-                                         strtol (elements[ne-1], NULL, 10),
-                                         local_path,
-                                         (LIBMTP_progressfunc_t)mtp_progress,
-                                         &mtp_progress_data);
-      if (ret != 0) {
-        fail_job (G_VFS_JOB (job), device);
-        goto exit;
-      }
+    MtpProgressData mtp_progress_data;
+    mtp_progress_data.progress_callback = progress_callback;
+    mtp_progress_data.progress_callback_data = progress_callback_data;
+    mtp_progress_data.job = G_VFS_JOB (job);
+    int ret = LIBMTP_Get_File_To_File (device,
+                                       strtol (elements[ne-1], NULL, 10),
+                                       local_path,
+                                       (LIBMTP_progressfunc_t)mtp_progress,
+                                       &mtp_progress_data);
+    if (ret != 0) {
+      fail_job (G_VFS_JOB (job), device);
+      goto exit;
+    }
+    /* Attempt to delete object if requested but don't fail it it fails. */
+    if (remove_source) {
+      DEBUG ("(I) Removing source.");
+      LIBMTP_Delete_Object (device, strtol (elements[ne-1], NULL, 10));
+    }
     g_vfs_job_succeeded (G_VFS_JOB (job));
   }
 
@@ -1317,6 +1328,12 @@ do_push (GVfsBackend *backend,
     goto exit;
   }
 
+  /* Attempt to delete object if requested but don't fail it it fails. */
+  if (remove_source) {
+    DEBUG ("(I) Removing source.");
+    g_file_delete (file, G_VFS_JOB (job)->cancellable, NULL);
+  }
+
   g_vfs_job_succeeded (G_VFS_JOB (job));
 
   g_hash_table_foreach (G_VFS_BACKEND_MTP (backend)->monitors,
